<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Shipping Module "Freight101"
 *
 * @package ShippingModuleFreight101
 * @author Egor V. Derevyankin
 */
class Shipping_Module_Freight101 extends pm_sm_api
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * ShippingModuleFreight101  constructor.
     */
    function Shipping_Module_Freight101()
    {
    }

	function getUID()
	{
		return "C6F2F268-7B71-4E86-B384-088CDEF6B903";
	}

    /**
     * @return Info about this shipping module
     */
    function getInfo()
    {

        global $application;
        $obj = &$application->getInstance('MessageResources',"shipping-module-freight101-messages", "AdminZone");

        $this->getSettings();

        return array("GlobalUniqueShippingModuleID" => $this->getUID(),
                     "Name"        => $obj->getMessage('MODULE_NAME'),
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$obj->getMessage('MODULE_STATUS_ACTIVE')."</span>":$obj->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>", //NOT for active/inactive.
                     "PreferencesAZViewClassName" => "freight101_input_az",
                     "APIClassName" => __CLASS__,
              );
    }

    /**
     * Checks the requirments.
     *
     * @return array with the failed requirments
     */
    function checkRequirments()
    {
        $return=array();

        //check SOAP library
        //if((version_compare(phpversion(),"5.0.0")==-1) or !class_exists("SoapClient"))
            //$return[]="SOAP";

        //check origin country
        $ocntr = modApiFunc("Location","getCountryCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_COUNTRY));
        if($ocntr!="US")
            $return[]="OCNTR";

        //check local currency
        $main_currency_id = modApiFunc("Localization", "getMainStoreCurrency");
        $local_currency_code = modApiFunc("Localization", "getCurrencyCodeById", $main_currency_id);
        if($local_currency_code!="USD")
            $return[]="CURRENCY";

        return $return;
    }

    /**
     * Module installator.
     */
    function install()
    {
        _use(dirname(__FILE__).'/includes/install.inc');

        //                                RemovePersonInfoType
        //                   ,          -     CheckoutEditor
        //             PersonInfoType,         , BillingInfo, ShippingInfo,
        //  CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
        //             ), ShippingMethodInfo (              ).
        //       ShippingModule'                                                 ,
        //                                     .
        //                    ,                 PersonInfoType          .
        //
        //  PersonInfoType.
        modApiFunc('EventsManager',
               'addEventHandler',
               'RemovePersonInfoTypeEvent',
               'Shipping_Module_Freight101',
               'OnRemovePersonInfoType');
    }

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
     *            ), ShippingMethodInfo (              ).
     *      ShippingModule'                                                 ,
     *                                    .
     *                   ,                 PersonInfoType          .
     *
     * PersonInfoType.
     *
     *        Freight101                         PersonInfoType:
     * shippingInfo
     *
     * @param string $person_info_type_tag - tag      person info type' ,
     *                      (turn off)   CheckoutFormEditor' .
     *
     * @return mixed - NULL,                       - "          ",       -
     *                                     ,            ,
     *        (                                   ).
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if($this->isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this->MessageResources = &$application->getInstance('MessageResources',"shipping-module-freight101-messages", "AdminZone");
            $msg = $this->MessageResources->getMessage('MODULE_SHIPPING_FREIGHT101_API_MSG_REMOVE_PERSON_INFO_TYPE');

            $value = NULL;
            switch($person_info_type_tag)
            {
                case "shippingInfo":
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }


    /**
     * @return true if module is active or false if is not
     */
    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules", "shipping");
        return array_key_exists($this->getUid(), $active_modules);
    }

    /**
     * Uninstalls the module.
     */
    function uninstall()
    {
        $query = new DB_Table_Delete(Shipping_Module_Freight101::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    /**
     * @param $method_id ID of shipping method of this module
     * @return true if ID is valid or false if is not
     */
    function isValidShippingMethodId($method_id)
    {
        /*
        global $application;
        $tables=$this->getTables();
        $methods_table=$tables['sm_freight101_methods']['columns'];

        $query = new DB_Select();
        $query->addSelectTable('sm_freight101_methods');
        $query->addSelectField($methods_table['id'],'id');
        $query->Where($methods_table['id'], DB_EQ, intval($method_id));

        $check_result=$application->db->getDB_Result($query);

        if(isset($check_result[0]['id']))
            return true;
        else
            return false;
        */
        return true;
    }

    /**
     * @param $method_id ID of shipping method of this module
     * @param $and_calc true if it is necessary to calculate the shipping cost
     *         or false if it is not
     * @return array with the shipping method info (id,method_name,method_code,
     *         destination,available,cost)
     */
    function getShippingMethodInfo($method_id,$and_calc=false)
    {
        $method=array(
            "id"            => $method_id,
            "method_name"   => "Freight101 #".$method_id,
            "method_code"   => "F101_".$method_id,
            "destination"   => "L",
            "available"     => "Y",
            "cost"          => PRICE_N_A
        );

        if($and_calc!=false)
        {
//            $ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
  //          $pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
            $ShippingInfo = modApiFunc("Shipping_Cost_Calculator","getShippingInfo");
            $pak_weight = modApiFunc("Shipping_Cost_Calculator","getPackWeight");
            $rate=$this->getRateFromCache($ShippingInfo,$pak_weight,$method_id);
            if($rate!=false)
                $method["cost"]=$rate;
        };

        return $method;
    }

    /**
     * @return structure of the tables of this module
     */
    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array ();

        $settings = 'sm_freight101_settings';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                => $settings.'.sm_freight101_setting_id'
               ,'key'               => $settings.'.sm_freight101_setting_key'
               ,'value'             => $settings.'.sm_freight101_setting_value'
            );
        $tables[$settings]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'key'               => DBQUERY_FIELD_TYPE_CHAR50
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        $rates_cache = 'sm_freight101_rates_cache';
        $tables[$rates_cache] = array();
        $tables[$rates_cache]['columns'] = array
            (
                'id'                => $rates_cache.'.sm_freight101_crate_id'
               ,'hash'              => $rates_cache.'.sm_freight101_crate_hash'
               ,'method_id'         => $rates_cache.'.sm_freight101_crate_method_id'
               ,'rate'              => $rates_cache.'.sm_freight101_crate_rate'
               ,'expire'            => $rates_cache.'.sm_freight101_crate_expire'
            );
        $tables[$rates_cache]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'hash'              => DBQUERY_FIELD_TYPE_CHAR50
               ,'method_id'         => DBQUERY_FIELD_TYPE_INT
               ,'rate'              => DBQUERY_FIELD_TYPE_DECIMAL20_5
               ,'expire'            => DBQUERY_FIELD_TYPE_INT
            );
        $tables[$rates_cache]['primary'] = array
            (
                'id'
            );


        global $application;
        return $application->addTablePrefix($tables);
    }

    /**
     *
     */
    function clearSettingsInDB()
    {
        global $application;
        $query = new DB_Delete('sm_freight101_settings');
        $application->db->getDB_Result($query);
    }

    /**
     * @return array of the settings of this module
     */
    function getSettings()
    {
        $result = execQuery('SELECT_PM_SM_SETTINGS',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'sm_freight101_settings'));
        $this->Settings = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
            $this->Settings[$result[$i]['set_key']] = unserialize($result[$i]['set_value']);
        }
        return $this->Settings;
    }

    /**
     * Sets settings for this module.
     *
     * @param $Settings array of all the settings of this module
     */
    function setSettings($Settings)
    {
        global $application;
        $this->clearSettingsInDB();
        $tables = $this->getTables();
        $columns = $tables['sm_freight101_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Insert('sm_freight101_settings');
            $query->addInsertValue($key, $columns['key']);
            $query->addInsertValue(serialize($value), $columns['value']);
            $application->db->getDB_Result($query);

            $inserted_id = $application->db->DB_Insert_Id();
        }
    }

    /**
     * Updates settings for this module.
     *
     * @param $Settings array of the updated settings of this module
     */
    function updateSettings($Settings)
    {

        global $application;
        $tables = $this->getTables();
        $columns = $tables['sm_freight101_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Update('sm_freight101_settings');
            $query->addUpdateValue($columns['value'], serialize($value));
            $query->WhereValue($columns['key'], DB_EQ, $key);
            $application->db->getDB_Result($query);
        }

    }

    /**
     * @param $type type of the requested methods (ALL|AVAILABLE|NOT_AVAILABLE)
     * @param $and_calc true if it is necessary to calculate the shipping cost
     *        or false if it is not
     * @return array of the shipping methods arrays, every will be as
     *        (id,method_name,method_code,destination,available,cost)
     */
    function getShippingMethods($type="ALL",$and_calc=false)
    {
        $methods=array();
        if($and_calc!=false)
        {

			/*
            if($this->is_debug==true)
            {
                $ShippingInfo = modApiFunc("Shipping_Tester","getShippingInfo");
                $pak_weight=modApiFunc("Shipping_Tester","getPackWeight");
            }
            else
            {
                $ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
                $pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
            }
            */

            $ShippingInfo = modApiFunc("Shipping_Cost_Calculator","getShippingInfo");
            $pak_weight=modApiFunc("Shipping_Cost_Calculator","getPackWeight");

            $_rates=array();

            if($ShippingInfo["isMet"]!=false)
            {
                $_rates=$this->calculate_rates($ShippingInfo,$pak_weight);
            };

            if(!empty($_rates))
            {
                foreach($_rates as $service => $sdata)
                {
                    $methods[]=array(
                        "id" => $service,
                        "method_name" => "Freight101 #".$service,
                        "method_code" => "F101_".$service,
                        "destination" => "L",
                        "available" => "Y",
                        "days" => $sdata["days"],
                        "cost" => $sdata["cost"]
                    );
                };
            };

            $this->saveRatesToCache($ShippingInfo,$pak_weight,$methods);
        };

        return $methods;
    }

    /**
     * @param $s_info shipping info of the destination
     * @param $weight weight of the package
     * @param $rates array of the shipping rates for saving
     */
    function saveRatesToCache($s_info,$weight,$rates)
    {
        if($s_info["isMet"]==false or empty($rates))
            return;

        $s_country=modApiFunc("Location","getCountry",$s_info["validatedData"]["Country"]["value"]);
        $s_zipcode=$s_info["validatedData"]["Postcode"]["value"];
        $s_name = isset($s_info["validatedData"]["Firstname"]["value"]) ? $s_info["validatedData"]["Firstname"]["value"] : '';
        $s_name.= isset($s_info["validatedData"]["Lastname"]["value"])  ? $s_info["validatedData"]["Lastname"]["value"]  : '';

        $hash=md5($s_country.$s_name.$s_zipcode.$weight);

        $tables=$this->getTables();
        $rc_table=$tables['sm_freight101_rates_cache']['columns'];

        global $application;

        $query = new DB_Delete('sm_freight101_rates_cache');
        $query->Where($rc_table['hash'], DB_EQ, "'$hash'");
        $application->db->getDB_Result($query);

        $query = new DB_Delete('sm_freight101_rates_cache');
        $query->Where($rc_table['expire'], DB_LT, time());
        $application->db->getDB_Result($query);

        foreach($rates as $key => $rate_info)
        {
            $query = new DB_Insert('sm_freight101_rates_cache');
            $query->addInsertValue($hash, $rc_table["hash"]);
            $query->addInsertValue($rate_info["id"],$rc_table["method_id"]);
            $query->addInsertValue($rate_info["cost"],$rc_table["rate"]);
            $query->addInsertValue(time()+18000,$rc_table["expire"]);
            $application->db->getDB_Result($query);
        };

        return;
    }

    /**
     * @param $s_info shipping info of the destination
     * @param $weight weight of the package
     * @param $method_id ID of the required shipping method
     * @return cost of the shipping from the cache
     *      or false if the requested cost is not saved in the cache
     */
    function getRateFromCache($s_info,$weight,$method_id)
    {
        if($s_info["isMet"]==false or !is_numeric($method_id))
            return false;

        $s_country=modApiFunc("Location","getCountry",$s_info["validatedData"]["Country"]["value"]);
        $s_zipcode=$s_info["validatedData"]["Postcode"]["value"];
        $s_name = isset($s_info["validatedData"]["Firstname"]["value"]) ? $s_info["validatedData"]["Firstname"]["value"] : '';
        $s_name.= isset($s_info["validatedData"]["Lastname"]["value"])  ? $s_info["validatedData"]["Lastname"]["value"]  : '';

        $hash=md5($s_country.$s_name.$s_zipcode.$weight);

        $tables=$this->getTables();
        $rc_table=$tables['sm_freight101_rates_cache']['columns'];

        global $application;

        $cached = execQuery('SELECT_SM_FREIGHT101_CACHED_RATES',array('ApiClassName' => __CLASS__, 'hash' => $hash, 'method_id' => $method_id));

        if(isset($cached[0]["rate"]))
            return $cached[0]["rate"];
        else
            return false;
    }

    /**
     * @param $ShippingInfo shipping info of the destination
     * @param $pak_weight weight of the package
     * @param $methods array of the available methods
     * @return array of the shipping rates arrays
     */
    function calculate_rates($ShippingInfo, $pak_weight)
    {
        if($this->is_debug==true)
            $this->_debug_info[]=array("Calculate Type","Real-Time by SOAP");

        $_rates = array();

        # get module settings
        $Settings=$this->getSettings();

        # get origin zip and destination zip and country code
        if($Settings["ORG_ZIP"]=="defined")
            $ozip = $Settings["DEFINED_OZIP"];
        else
            $ozip = modApiFunc("Configuration","getTagValue","StoreOwnerPostcode");
        $dzip = $ShippingInfo["validatedData"]["Postcode"]["value"];
        $dcntr = modApiFunc("Location","getCountryCode",$ShippingInfo["validatedData"]["Country"]["value"]);

        if($dcntr!="US")
        {
            if($this->is_debug==true)
            {
                $this->_debug_info[]=array("Error","Invalid destination country");
            };
            return $_rates;
        }

        # calculate weight in lbs
        $local_wcoeff = modApiFunc("Localization","getValue","WEIGHT_COEFF");
        $weight_in_lbs = round($pak_weight * $local_wcoeff * 1000 / 453.6);
        if($weight_in_lbs < 1)
            $weight_in_lbs = 1;

        if($this->is_debug==true)
        {
            $this->_debug_info[]=array("Origination Zip",$ozip);
            $this->_debug_info[]=array("Destination Zip",$dzip);
            $this->_debug_info[]=array("Service URL is","http://api.freight101.com/freight101.php");
        }

        loadCoreFile('bouncer.php');
        global $application;
        $bnc = new Bouncer();
        $bnc->setMethod("POST");
        $bnc->setHTTPversion($application->getAppIni('FREIGHT101_HTTP_PROTOCOL_VERSION'));
        $bnc->setURL($application->getAppIni('FREIGHT101_SERVER_URL'));
        $bnc->setPOSTtype("text/xml; charset=utf-8");

        $bnc->setAdditionalHeaders(array("SOAPAction"=>"\"http://api.freight101.com/freight101.php#Authenticate\""));

        $post_data="<?xml version=\"1.0\" encoding=\"UTF-8\"?>".
                        "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" ".
                         "xmlns:ns1=\"http://api.freight101.com/\">".
                            "<SOAP-ENV:Body>".
                                "<ns1:APIKey>".$Settings["AUTH_KEY"]."</ns1:APIKey>".
                            "</SOAP-ENV:Body>".
                        "</SOAP-ENV:Envelope>";

        if($this->is_debug==true)
            $this->_debug_info[]=array("Authenticate SOAP XML request",$post_data);

        $bnc->setPOSTstring($post_data);
        $result=$bnc->RunRequest();

        if($result==false)
        {
            if($this->is_debug==true)
                $this->_debug_info[]=array("Error",$bnc->getLastErrorMessage());
            return $_rates;
        }

        if(!empty($result["cookies"]["valid"]))
            $bnc->setCookies($result["cookies"]["valid"]);

        $bnc->setAdditionalHeaders(array("SOAPAction"=>"\"http://api.freight101.com/freight101.php#setConnectionType\""));

        $post_data="<?xml version=\"1.0\" encoding=\"UTF-8\"?>".
                        "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" ".
                         "xmlns:ns1=\"http://api.freight101.com/\">".
                            "<SOAP-ENV:Body>".
                                "<ns1:ConnectionTypeRequest>php</ns1:ConnectionTypeRequest>".
                            "</SOAP-ENV:Body>".
                        "</SOAP-ENV:Envelope>";

        if($this->is_debug==true)
            $this->_debug_info[]=array("Set Connection type SOAP XML request",$post_data);

        $bnc->setPOSTstring($post_data);
        $result=$bnc->RunRequest();

        if($result==false)
        {
            if($this->is_debug==true)
                $this->_debug_info[]=array("Error",$bnc->getLastErrorMessage());
            return $_rates;
        }

        $bnc->setAdditionalHeaders(array("SOAPAction"=>"\"http://api.freight101.com/freight101.php#setOrigin\""));

        $post_data="<?xml version=\"1.0\" encoding=\"UTF-8\"?>".
                    "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" ".
                    "xmlns:ns1=\"http://api.freight101.com/\">".
                        "<SOAP-ENV:Body>".
                            "<ns1:OriginRequest>".
                                "<ns1:zip>".$ozip."</ns1:zip>".
                                "<ns1:trade_show>".($Settings["ORG_TS"]=="YES"?"true":"false")."</ns1:trade_show>".
                                "<ns1:lift_gate>".($Settings["ORG_LGN"]=="YES"?"true":"false")."</ns1:lift_gate>".
                                "<ns1:inside_delivery>".($Settings["ORG_IP"]=="YES"?"true":"false")."</ns1:inside_delivery>".
                                "<ns1:appt_required>".($Settings["ORG_AN"]=="YES"?"true":"false")."</ns1:appt_required>".
                                "<ns1:location_type>".$Settings["ORG_LT"]."</ns1:location_type>".
                            "</ns1:OriginRequest>".
                        "</SOAP-ENV:Body>".
                    "</SOAP-ENV:Envelope>";

        if($this->is_debug==true)
            $this->_debug_info[]=array("Set Origin SOAP XML request",$post_data);

        $bnc->setPOSTstring($post_data);
        $result=$bnc->RunRequest();

        if($result==false)
        {
            if($this->is_debug==true)
                $this->_debug_info[]=array("Error",$bnc->getLastErrorMessage());
            return $_rates;
        }

        $bnc->setAdditionalHeaders(array("SOAPAction"=>"\"http://api.freight101.com/freight101.php#setDestionation\""));

        $post_data="<?xml version=\"1.0\" encoding=\"UTF-8\"?>".
                    "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" ".
                    "xmlns:ns1=\"http://api.freight101.com/\">".
                        "<SOAP-ENV:Body>".
                            "<ns1:DestinationRequest>".
                                "<ns1:zip>".$dzip."</ns1:zip>".
                                "<ns1:trade_show>".($Settings["DST_TS"]=="YES"?"true":"false")."</ns1:trade_show>".
                                "<ns1:lift_gate>".($Settings["DST_LGN"]=="YES"?"true":"false")."</ns1:lift_gate>".
                                "<ns1:inside_delivery>".($Settings["DST_IP"]=="YES"?"true":"false")."</ns1:inside_delivery>".
                                "<ns1:appt_required>".($Settings["DST_AN"]=="YES"?"true":"false")."</ns1:appt_required>".
                                "<ns1:location_type>".$Settings["DST_LT"]."</ns1:location_type>".
                            "</ns1:DestinationRequest>".
                        "</SOAP-ENV:Body>".
                    "</SOAP-ENV:Envelope>";

        if($this->is_debug==true)
            $this->_debug_info[]=array("Set Destination SOAP XML request",$post_data);

        $bnc->setPOSTstring($post_data);
        $result=$bnc->RunRequest();

        if($result==false)
        {
            if($this->is_debug==true)
                $this->_debug_info[]=array("Error",$bnc->getLastErrorMessage());
            return $_rates;
        }

        $bnc->setAdditionalHeaders(array("SOAPAction"=>"\"http://api.freight101.com/freight101.php#setPackage\""));

        $post_data="<?xml version=\"1.0\" encoding=\"UTF-8\"?>".
                    "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" ".
                    "xmlns:ns1=\"http://api.freight101.com/\">".
                        "<SOAP-ENV:Body>".
                            "<ns1:PackageRequest>".
                                "<ns1:class>".$Settings["PCLASS"]."</ns1:class>".
                                "<ns1:weight>".$weight_in_lbs."</ns1:weight>".
                                "<ns1:hazmat>".($Settings["PCK_HAZ"]=="YES"?"true":"false")."</ns1:hazmat>".
                            "</ns1:PackageRequest>".
                        "</SOAP-ENV:Body>".
                    "</SOAP-ENV:Envelope>";

        if($this->is_debug==true)
            $this->_debug_info[]=array("Set Package SOAP XML request",$post_data);

        $bnc->setPOSTstring($post_data);
        $result=$bnc->RunRequest();

        if($result==false)
        {
            if($this->is_debug==true)
                $this->_debug_info[]=array("Error",$bnc->getLastErrorMessage());
            return $_rates;
        }

        $bnc->setAdditionalHeaders(array("SOAPAction"=>"\"http://api.freight101.com/freight101.php#getQuote\" "));

        $post_data="<?xml version=\"1.0\" encoding=\"UTF-8\"?>".
                    "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" ".
                    "xmlns:ns1=\"http://api.freight101.com/\">".
                        "<SOAP-ENV:Body>".
                            "<ns1:QuoteRequest/>".
                        "</SOAP-ENV:Body>".
                    "</SOAP-ENV:Envelope>";

        if($this->is_debug==true)
            $this->_debug_info[]=array("Get Quote SOAP XML request",$post_data);

        $bnc->setPOSTstring($post_data);
        $result=$bnc->RunRequest();

        if($result==false)
        {
            if($this->is_debug==true)
                $this->_debug_info[]=array("Error",$bnc->getLastErrorMessage());
            return $_rates;
        }

        $xml_response=$result["body"];

        if($this->is_debug==true)
            $this->_debug_info[]=array("Get Quote SOAP XML response",$xml_response);


        loadCoreFile('obj_xml.php');
        $parser = new xml_doc($xml_response);
        $parser->parse();

        $_rates=array();
        $_errors=array();
        $key=0;

        //print_r($parser->xml_index);

        foreach($parser->xml_index as $tag_key => $tag_obj)
        {
            if($tag_obj->name=="SOAP-ENV:FAULT")
            {
                $faultcode="";
                $faultstring="";
                foreach($tag_obj->children as $fault_tag_key => $fault_tag_obj)
                    if($fault_tag_obj->name=="FAULTCODE")
                        $faultcode=$fault_tag_obj->contents;
                    elseif($fault_tag_obj->name=="FAULTSTRING")
                        $faultstring=$fault_tag_obj->contents;
                $_errors[]=array("code"=>$faultcode,"string"=>$faultstring);
            }
            elseif($tag_obj->name=="NS1:RATES")
            {
                foreach($tag_obj->children as $rates_tag_key => $rates_tag_obj)
                    if($rates_tag_obj->name=="NS1:RATE")
                    {
                        $cost=0;
                        $days=0;
                        foreach($rates_tag_obj->children as $rate_tag_key => $rate_tag_obj)
                            if($rate_tag_obj->name=="NS1:TOTAL_CHARGE")
                                $cost=$rate_tag_obj->contents;
                            elseif($rate_tag_obj->name=="NS1:SERVICE_DAYS")
                                $days=$rate_tag_obj->contents;
                        $_rates[++$key]=array("cost"=>$cost,"days"=>$days);
                    }
            }

        }

        if(!empty($_errors) and $this->is_debug==true)
            foreach($_errors as $k => $error_data)
                $this->_debug_info[]=array("Error",$error_data["code"].": ".$error_data["string"]);

        return $_rates;
    }

    function setDebugFlag($flag=true)
    {
        if(is_bool($flag))
            $this->is_debug=$flag;
    }

    function getDebugInfo()
    {
        return $this->_debug_info;
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
	 * @deprecated
     * @param $method_id ID of shipping method of this module
     * @return shipping cost or PRICE_N_A
     */
    function getShippingCost($method_id="")
    {
        $ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
        $pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
        $rate=$this->getRateFromCache($ShippingInfo,$pak_weight,$method_id);
        if($rate!=false)
            return $rate;
        else
            return PRICE_N_A;
    }

    var $bActive;

    var $is_debug;
    var $_debug_info;

    /**#@-*/
}
?>